@@ -23,11 +23,13 @@
 #ifndef BISECTINGKMEANS_H
 #define BISECTINGKMEANS_H
 
-#include "global.h"
-#include "msa.h"
-#include "rng.h"
 #include "alignment_parameters.h"
-#include "sequence_distance.h"
 
-extern int build_tree_kmeans(struct msa* msa, struct aln_param* ap);
+struct aln_tasks;
+
+int build_tree_kmeans(struct msa* msa, struct aln_param* ap,struct  aln_tasks** task_list);
+
+//int build_tree_kmeans(struct msa* msa, struct aln_param* ap);
+
+//extern int build_tree_kmeans(struct msa* msa, struct aln_param* ap);
 #endif
