@@ -20,11 +20,24 @@
 
 */
 
+
+#include "tlmisc.h"
+
 #include "global.h"
 #include "msa.h"
 #include "alphabet.h"
 #include "esl_stopwatch.h"
 
+#include <string.h>
+#include <ctype.h>
+
+struct sort_struct_name_chksum{
+        char** name;
+        int chksum;
+        int action;
+};
+
+
 struct in_line{
         char* line;
         int len;
@@ -53,9 +66,9 @@ struct out_line{
 
 
 
-struct msa* read_fasta(struct in_buffer* b, struct msa* msa);
-struct msa* read_msf(struct in_buffer* b, struct msa* msa);
-struct msa* read_clu(struct in_buffer* b, struct msa* msa);
+int read_fasta(struct in_buffer* b, struct msa** msa);
+int read_msf(struct in_buffer* b, struct msa** msa);
+int read_clu(struct in_buffer* b, struct msa** msa);
 
 int write_msa_fasta(struct msa* msa,char* outfile);
 int write_msa_clustal(struct msa* msa,char* outfile);
@@ -89,11 +102,15 @@ static int set_sip_nsip(struct msa* msa);
 static int null_terminate_sequences(struct msa* msa);
 static int sort_out_lines(const void *a, const void *b);
 static int make_linear_sequence(struct msa_seq* seq, char* linear_seq);
-int GCGMultchecksum(struct msa* msa);
+static int GCGMultchecksum(struct msa* msa);
 /* Taken from squid library by Sean Eddy  */
-int GCGchecksum(char *seq, int len);
+static int GCGchecksum(char *seq, int len);
 
 
+
+static int sort_by_name(const void *a, const void *b);
+static int sort_by_chksum(const void *a, const void *b);
+
 #ifdef RWALIGN_TEST
 int print_msa(struct msa* msa);
 int main(int argc, char *argv[])
@@ -113,7 +130,7 @@ int main(int argc, char *argv[])
         }
         LOG_MSG("reading: %s", buffer);
 
-        RUNP(msa = read_input(buffer,NULL));
+        RUN(read_input(buffer,&msa));
         if(msa->aligned == ALN_STATUS_UNKNOWN){
                 RUN(dealign_msa(msa));
         }
@@ -160,11 +177,12 @@ int print_msa(struct msa* msa)
 }
 #endif
 
-struct msa* read_input(char* infile,struct msa* msa)
+int read_input(char* infile,struct msa** msa)
 {
         struct in_buffer* b = NULL;
+        struct msa* m = NULL;
         int type;
-
+        int i,j;
         //ASSERT(infile != NULL,"No input file");
         /* sanity checks  */
         if(infile){
@@ -180,37 +198,63 @@ struct msa* read_input(char* infile,struct msa* msa)
 
         /* read everything into a in_buffer  */
 
-
         RUN(read_file_stdin(&b,infile));
-        //exit(0);
+
+        /* Check if any input was read */
+        /* Logic: sum length of lines 1.. up to 5 */
+        //LOG_MSG("%s lines: %d", infile, b->n_lines);
+        j = 0;
+        for(i = 0; i < MACRO_MIN(1, b->n_lines);i++){
+                j += b->l[i]->len -1; /* exclude '0' at the end of strings  */
+                //fprintf(stdout,"%d %s\n", i, b->l[i]->line);
+        }
+        //LOG_MSG("Len: %d",j);
+        if(j == 0){
+                DESTROY_TIMER(timer);
+                free_in_buffer(b);
+                *msa = NULL;
+                return OK;
+        }
 
         RUN(detect_alignment_format(b, &type));
-        //exit(0);
-
 
+        /* LOG_MSG("FORMAT: %d", type); */
         if(type == FORMAT_FA){
                 //RUNP(msa = read_msf(infile,msa));
                 //RUNP(msa = read_clu(infile,msa));
-                RUNP(msa = read_fasta(b,msa));
+                RUN(read_fasta(b,&m));
         }else if(type == FORMAT_MSF){
-                RUNP(msa = read_msf(b,msa));
+                RUN(read_msf(b,&m));
         }else if(type == FORMAT_CLU){
-                RUNP(msa = read_clu(b,msa));
+                RUN(read_clu(b,&m));
+        }else if(type == FORMAT_DETECT_FAIL){
+                if(infile){
+                        WARNING_MSG("Could not detect input in file: %s", infile);
+                }else{
+                        WARNING_MSG("Could not detect input in standard input");
+                }
+                /* clean up allocated structures */
+                free_in_buffer(b);
+                DESTROY_TIMER(timer);
+                *msa = NULL;
+                return OK;
         }
 
-        RUN(detect_alphabet(msa));
-        RUN(detect_aligned(msa));
+        RUN(detect_alphabet(m));
+        RUN(detect_aligned(m));
 
-        RUN(set_sip_nsip(msa));
+        RUN(set_sip_nsip(m));
         free_in_buffer(b);
         STOP_TIMER(timer);
         GET_TIMING(timer);
         DESTROY_TIMER(timer);
         //LOG_MSG("Done reading input sequences in %f seconds.", GET_TIMING(timer));
-        return msa;
+        *msa = m;
+        return OK;
 ERROR:
-        free_msa(msa);
-        return NULL;
+
+        free_msa(m);
+        return FAIL;
 }
 
 int write_msa(struct msa* msa, char* outfile, int type)
@@ -227,7 +271,6 @@ int write_msa(struct msa* msa, char* outfile, int type)
         }else{
                 ERROR_MSG("Output format not recognized.");
         }
-
         return OK;
 ERROR:
         return FAIL;
@@ -311,10 +354,12 @@ int detect_alignment_format(struct in_buffer*b,int* type)
                 }
         }
         if(set == 0){
-                ERROR_MSG("Input alignment format could not be detected.");
+                *type = FORMAT_DETECT_FAIL;
+                //ERROR_MSG("Input alignment format could not be detected.");
         }
         if(set > 1){
-                ERROR_MSG("Input format could not be unambiguously detected");
+                *type = FORMAT_DETECT_FAIL;
+                //ERROR_MSG("Input format could not be unambiguously detected");
         }
         if(hints[0]){
                 *type = FORMAT_FA;
@@ -337,60 +382,64 @@ ERROR:
 int detect_alphabet(struct msa* msa)
 {
         int i;
-        int min,c;
-        uint8_t DNA[128];
-        uint8_t protein[128];
-        int diff[3];
-        char DNA_letters[]= "acgtuACGTUnN";
+
+        double DNA[128];
+        double protein[128];
+        char DNA_letters[]= "acgtunACGTUN";
         char protein_letters[] = "acdefghiklmnpqrstvwyACDEFGHIKLMNPQRSTVWY";
 
+        double dna_prob;
+        double prot_prob;
+
         ASSERT(msa != NULL, "No alignment");
 
-        for(i = 0; i <128;i++){
-                DNA[i] = 0;
-                protein[i] = 0;
+        for(i = 0; i < 128;i++){
+                DNA[i] = log(0.0001 * 1.0 / 116.0);
+                protein[i] = log(0.0001 * 1.0 / 88.0);
         }
 
         for(i = 0 ; i < (int)strlen(DNA_letters);i++){
-                DNA[(int) DNA_letters[i]] = 1;
+                DNA[(int) DNA_letters[i]] = log(0.9999 * 1.0 / 12.0);
         }
 
         for(i = 0 ; i < (int)strlen(protein_letters);i++){
-                protein[(int) protein_letters[i]] = 1;
+                protein[(int) protein_letters[i]] = log(0.9999 * 1.0 / 40.0);
         }
+        /* dna_prob = 0.0; */
+        /* prot_prob = 0.0; */
+        /* for(i = 0; i <128;i++){ */
+        /*         dna_prob += exp(DNA[i]); */
 
-        diff[0] = 0;
-        diff[1] = 0;
+        /*         prot_prob += exp(protein[i]); */
+        /* } */
+        /* LOG_MSG("DNA: %f PROT: %f",dna_prob,prot_prob); */
+
+        dna_prob = 0.0;
+        prot_prob = 0.0;
         for(i = 0; i < 128;i++){
-                if((msa->letter_freq[i]) && (!DNA[i])){
-                        diff[0]++;
-                }
-                if((msa->letter_freq[i]) && (!protein[i])){
-                        diff[1]++;
+                if(msa->letter_freq[i]){
+                        dna_prob += DNA[i] * (double) msa->letter_freq[i];
+                        prot_prob += protein[i]* (double) msa->letter_freq[i];
                 }
         }
 
-        if( diff[0] + diff[1] == 0){
-                ERROR_MSG("Could not detect any AA or nucleotides.");
-        }
-        c = -1;
-        min = 2147483647;
-        for(i = 0; i < 2;i++){
-                if(diff[i] < min){
-                        min = diff[i];
-                        c = i;
-                }
-        }
-        if(c == 0){
-                LOG_MSG("Detected DNA sequences.");
-                msa->L = defDNA;
-                RUN(convert_msa_to_internal(msa, defDNA));
-        }else if(c == 1){
-                LOG_MSG("Detected protein sequences.");
-                msa->L = redPROTEIN;
-                RUN(convert_msa_to_internal(msa, redPROTEIN));
+        /* LOG_MSG("DNA: %f PROT: %f",dna_prob,prot_prob); */
+        /* exit(0); */
+        if( dna_prob == prot_prob){
+                WARNING_MSG("Could not determine whether we have a DNA or Protein alignment");
+                msa->L = ALPHA_UNKNOWN;
         }else{
-                ERROR_MSG("Alphabet not recognized.");
+                if(dna_prob > prot_prob){
+                        LOG_MSG("Detected DNA sequences.");
+                        msa->L = ALPHA_defDNA;
+                        RUN(convert_msa_to_internal(msa, ALPHA_defDNA));
+                }else if(prot_prob > dna_prob){
+                        LOG_MSG("Detected protein sequences.");
+                        msa->L = ALPHA_redPROTEIN;
+                        RUN(convert_msa_to_internal(msa, ALPHA_redPROTEIN));
+                }else{
+                        ERROR_MSG("Alphabet not recognized.");
+                }
         }
         return OK;
 ERROR:
@@ -399,48 +448,142 @@ ERROR:
 
 int detect_aligned(struct msa* msa)
 {
-        /* Here I look for gap character in the character frequency vector
-           if they are there we know the input sequences were aligned. This is not strictly
-           necessary for .clu and .msf infiles - but better safe than sorry. */
         int min_len;
         int max_len;
+        int l;
         int i;
+        int j;
+        int n;
         int gaps = 0;
         /* assume that sequences are not aligned */
         msa->aligned = 0;
 
-        gaps += msa->letter_freq[ (int) '-'];
-        gaps += msa->letter_freq[ (int) '.'];
-        gaps += msa->letter_freq[ (int) '_'];
-        gaps += msa->letter_freq[ (int) '~'];
-
-
-
-        /* if all sequences are the same length they could be considered
-           aligned (at least for the purpose of writing out a .msf / .clu
-           file) */
+        /* Improved logic:
+           Lets sum up the number of gaps plus sequence length of the first
+           X sequences. if min == max it is aligned.
+        */
         min_len = INT32_MAX;
-        max_len = INT32_MIN;
-        for(i = 0; i < msa->numseq;i++){
-                min_len = MACRO_MIN(min_len, msa->sequences[i]->len);
-                max_len = MACRO_MAX(max_len, msa->sequences[i]->len);
+        max_len = 0;
+        gaps = 0;
+        n = MACRO_MIN(50, msa->numseq);
+        for(i = 0; i < n;i++){
+                l = 0;
+                for (j = 0; j <= msa->sequences[i]->len;j++){
+                        l += msa->sequences[i]->gaps[j];
+                }
+                gaps += l;
+                l += msa->sequences[i]->len;
+                min_len = MACRO_MIN(min_len, l);
+                max_len = MACRO_MAX(max_len, l);
         }
-        //LOG_MSG("%d %d",min_len,max_len);
-        if(min_len == max_len){
-                msa->aligned = 1;
-        }
-        if(!gaps && min_len == max_len){
-                msa->aligned = ALN_STATUS_ALIGNED;
-        }else if(!gaps && min_len != max_len){
-                msa->aligned = ALN_STATUS_UNALIGNED;
+        if(gaps){
+                if(min_len == max_len){ /* sequences have gaps and total length is identical - clearly aligned  */
+                        msa->aligned = ALN_STATUS_ALIGNED;
+                }else{          /* odd there are gaps but total length differs - unknown status  */
+                        msa->aligned = ALN_STATUS_UNKNOWN;
+                }
         }else{
-                msa->aligned = ALN_STATUS_UNKNOWN;
+                if(min_len == max_len){ /* no gaps and sequences have same length. Can' tell if they are aligned  */
+                        msa->aligned = ALN_STATUS_UNKNOWN;
+                }else{          /* No gaps and sequences have different lengths - unaligned */
+                        msa->aligned = ALN_STATUS_UNALIGNED;
+                }
         }
-
         //LOG_MSG("Aligned: %d gaps: %d",msa->aligned,gaps);
         return OK;
 }
 
+/* Checks if sequence names are duplicated */
+/* Checks if sequences are duplicated */
+int run_extra_checks_on_msa(struct msa* msa)
+{
+        char* tmp_name = NULL;
+        char* tmp_ptr;
+        struct sort_struct_name_chksum** a = NULL;
+        int i;
+        int j;
+        int c;
+        int l;
+
+        MMALLOC(a, sizeof(struct sort_struct_name_chksum *) * msa->numseq);
+
+        for(i = 0; i < msa->numseq;i++){
+                a[i] = NULL;
+                MMALLOC(a[i], sizeof(struct sort_struct_name_chksum));
+                a[i]->name = &msa->sequences[i]->name;
+                a[i]->chksum = GCGchecksum(msa->sequences[i]->seq, msa->sequences[i]->len);
+                a[i]->action = 0;
+        }
+
+        qsort(a, msa->numseq, sizeof(struct sort_struct*),sort_by_name);
+
+        for(i = 1; i < msa->numseq;i++){
+                if(strncmp(*a[i-1]->name, *a[i]->name,MSA_NAME_LEN) == 0){
+                        /* WARNING_MSG("Name: %s is duplicated", a[i]->name); */
+                        if(a[i-1]->chksum == a[i]->chksum){
+                                LOG_MSG("Found duplicated sequence:\n%s checksum: %d\n%s checksum: %d\n",
+                                        *a[i-1]->name,
+                                        a[i-1]->chksum,
+                                        *a[i]->name,
+                                        a[i]->chksum);
+                                a[i-1]->action = 1;
+                                a[i]->action = 1;
+                        }else{
+                                WARNING_MSG("Found sequence pair with same name but different sequence:\n%s checksum: %d\n%s checksum: %d\n",
+                                            *a[i-1]->name,
+                                            a[i-1]->chksum,
+                                            *a[i]->name,
+                                            a[i]->chksum);
+
+                                a[i-1]->action = 1;
+                                a[i]->action = 1;
+                                WARNING_MSG("This is a problem if we want to compare alignments down the track. Will append \"_X\" to the sequence name.");
+                        }
+                }
+        }
+
+        c = 1;
+        for(i = 0; i < msa->numseq;i++){
+                if(a[i]->action){
+
+                        tmp_name = NULL;
+                        l = strnlen(*a[i]->name, MSA_NAME_LEN)+ 16;
+                        MMALLOC(tmp_name, sizeof(char) * l);
+
+                        snprintf(tmp_name, l, "%s_%d",*a[i]->name,c);
+
+                        MFREE(*a[i]->name);
+                        *a[i]->name = tmp_name;
+                        c++;
+                }
+
+        }
+
+
+        qsort(a, msa->numseq, sizeof(struct sort_struct*),sort_by_chksum);
+        for(i = 1; i < msa->numseq;i++){
+                if(a[i-1]->chksum == a[i]->chksum){
+                        WARNING_MSG("Found identical sequences:\n%s checksum: %d\n%s checksum: %d\n",
+                                    *a[i-1]->name,
+                                    a[i-1]->chksum,
+                                    *a[i]->name,
+                                    a[i]->chksum);
+                }
+        }
+        for(i = 0; i < msa->numseq;i++){
+                MFREE(a[i]);
+        }
+        MFREE(a);
+        return OK;
+ERROR:
+        if(a){
+                for(i = 0; i < msa->numseq;i++){
+                        MFREE(a[i]);
+                }
+                MFREE(a);
+        }
+        return FAIL;
+}
 
 int dealign_msa(struct msa* msa)
 {
@@ -525,6 +668,7 @@ int convert_msa_to_internal(struct msa* msa, int type)
                         if(t[(int) seq->seq[j]] == -1){
                                 WARNING_MSG("there should be no character not matching the alphabet");
                                 WARNING_MSG("offending character: >>>%c<<<", seq->seq[j]);
+                                /* exit(0); */
                         }else{
                                 seq->s[j] = t[(int) seq->seq[j]];
                         }
@@ -600,9 +744,11 @@ int write_msa_msf(struct msa* msa,char* outfile)
         struct out_line* ol= NULL;
         struct msa_seq* seq = NULL;
         time_t now;			/* current time as a time_t */
+        struct tm local_time;
         char   date[64];		/* today's date in GCG's format "October 3, 1996 15:57" */
         char* linear_seq = NULL;
         char* ptr;
+        char* basename = NULL;
         FILE* f_ptr = NULL;
 
         int aln_len;
@@ -611,6 +757,7 @@ int write_msa_msf(struct msa* msa,char* outfile)
         int max_name_len = 0;
         int line_length;
         int header_index;
+        int written;
 
         if(!msa->aligned){
                 ERROR_MSG("Sequences appear to be unaligned!");
@@ -641,8 +788,9 @@ int write_msa_msf(struct msa* msa,char* outfile)
            1 for newline character +
            6 for spaces every 10 letters
         */
+        /* BUG: the header lines might be longer!!  */
         line_length = max_name_len +5 + 60 + 2+6 ;
-
+        line_length = MACRO_MAX(256, line_length);
         RUNP(lb = alloc_line_buffer(line_length));
 
         /* print header line
@@ -662,15 +810,16 @@ int write_msa_msf(struct msa* msa,char* outfile)
         header_index = -1 * (msa->numseq+10);
         ol = lb->lines[lb->num_line];
         //LOG_MSG("Alphabet : %d", msa->L);
-        if(msa->L == defPROTEIN){
+        if(msa->L == ALPHA_defPROTEIN){
                 snprintf(ol->line, line_length,"!!AA_MULTIPLE_ALIGNMENT 1.0");
-        }else if(msa->L == redPROTEIN){
+        }else if(msa->L == ALPHA_redPROTEIN){
                 snprintf(ol->line, line_length,"!!AA_MULTIPLE_ALIGNMENT 1.0");
-        }else if(msa->L == defDNA){
+        }else if(msa->L == ALPHA_defDNA){
                 snprintf(ol->line, line_length,"!!NA_MULTIPLE_ALIGNMENT 1.0");
         }else{
                 snprintf(ol->line, line_length,"!!NA_MULTIPLE_ALIGNMENT 1.0");
         }
+
         ol->block = -1;
         ol->seq_id = header_index;
         header_index++;
@@ -685,11 +834,31 @@ int write_msa_msf(struct msa* msa,char* outfile)
 
         /* The msf line*/
         now = time(NULL);
-        if (strftime(date, 64, "%B %d, %Y %H:%M", localtime(&now)) == 0){
+
+        if((localtime_r(&now,&local_time)) == NULL){
+                ERROR_MSG("could not get local time");
+        }
+
+        if (strftime(date, 64, "%B %d, %Y %H:%M", &local_time) == 0){
                 ERROR_MSG("time failed???");
         }
         ol = lb->lines[lb->num_line];
-        snprintf(ol->line, line_length," %s  MSF: %d  Type: %c  %s  Check: %d  ..", outfile == NULL ? "kalign" : outfile,aln_len, msa->L == defPROTEIN ? 'P' : 'N', date, GCGMultchecksum(msa));
+        if(outfile){
+                RUN(tlfilename(outfile, &basename));
+        }
+
+        written = snprintf(ol->line, line_length," %s  MSF: %d  Type: %c  %s  Check: %d  ..", outfile == NULL ? "stdout" :   basename,aln_len, msa->L == ALPHA_defPROTEIN ? 'P' : 'N', date, GCGMultchecksum(msa));
+
+        if(written >= line_length){
+                MREALLOC(lb->lines[lb->num_line]->line,sizeof(char) * (written+1));
+                ol = lb->lines[lb->num_line];
+                written = snprintf(ol->line, written+1," %s  MSF: %d  Type: %c  %s  Check: %d  ..", outfile == NULL ? "stdout" : basename,aln_len, msa->L == ALPHA_defPROTEIN ? 'P' : 'N', date, GCGMultchecksum(msa));
+
+        }
+
+        if(basename){
+                MFREE(basename);
+        }
 
         ol->block = -1;
         ol->seq_id = header_index;
@@ -712,12 +881,23 @@ int write_msa_msf(struct msa* msa,char* outfile)
 
                 ol = lb->lines[lb->num_line];
 
-                snprintf(ol->line, line_length," Name: %-*.*s  Len:  %5d  Check: %4d  Weight: %.2f",
-                         max_name_len,max_name_len,
-                         msa->sequences[i]->name ,
-                         aln_len,
-                         GCGchecksum(msa->sequences[i]->seq, msa->sequences[i]->len),
-                         1.0);
+                written = snprintf(ol->line, line_length," Name: %-*.*s  Len:  %5d  Check: %4d  Weight: %.2f",
+                                   max_name_len,max_name_len,
+                                   msa->sequences[i]->name ,
+                                   aln_len,
+                                   GCGchecksum(msa->sequences[i]->seq, msa->sequences[i]->len),
+                                   1.0);
+                if(written >= line_length){
+                        MREALLOC(lb->lines[lb->num_line]->line,sizeof(char) * (written+1));
+                        ol = lb->lines[lb->num_line];
+                        written = snprintf(ol->line, written+1," Name: %-*.*s  Len:  %5d  Check: %4d  Weight: %.2f",
+                                           max_name_len,max_name_len,
+                                           msa->sequences[i]->name ,
+                                           aln_len,
+                                           GCGchecksum(msa->sequences[i]->seq, msa->sequences[i]->len),
+                                           1.0);
+                }
+
 
                 ol->block = -1;
                 ol->seq_id = header_index;
@@ -817,8 +997,6 @@ int write_msa_msf(struct msa* msa,char* outfile)
                                 break;
                         }
                 }
-
-
         }
 
 
@@ -835,6 +1013,12 @@ int write_msa_msf(struct msa* msa,char* outfile)
         MFREE(linear_seq);
         return OK;
 ERROR:
+        if(linear_seq){
+                MFREE(linear_seq);
+        }
+        if(basename){
+                MFREE(basename);
+        }
         return FAIL;
 }
 
@@ -868,12 +1052,14 @@ int write_msa_clustal(struct msa* msa,char* outfile)
                 max_name_len = MACRO_MAX(max_name_len, (int)strnlen( msa->sequences[i]->name,MSA_NAME_LEN));
         }
 
+
         aln_len = 0;
         for (j = 0; j <= msa->sequences[0]->len;j++){
                 aln_len += msa->sequences[0]->gaps[j];
         }
         aln_len += msa->sequences[0]->len;
-        LOG_MSG("%d", aln_len);
+
+
         MMALLOC(linear_seq, sizeof(char)* (aln_len+1));
 
 
@@ -883,7 +1069,7 @@ int write_msa_clustal(struct msa* msa,char* outfile)
            1 for newline character
         */
         line_length = max_name_len +5 + 60 + 2;
-
+        line_length = MACRO_MAX(256, line_length );
         RUNP(lb = alloc_line_buffer(line_length));
 
         /* print header line
@@ -984,9 +1170,9 @@ ERROR:
         return FAIL;
 }
 
-struct msa* read_clu(struct in_buffer* b , struct msa* msa)
+int read_clu(struct in_buffer* b , struct msa** m)
 {
-        //struct msa* msa = NULL;
+        struct msa* msa = NULL;
         struct msa_seq* seq_ptr = NULL;
         //FILE* f_ptr = NULL;
         char* line = NULL;
@@ -1019,7 +1205,7 @@ struct msa* read_clu(struct in_buffer* b , struct msa* msa)
                 ni++;
                 //line_len = nread;
                 //line[line_len-1] = 0;
-                line_len--;
+                /* line_len--; */
                 break;
         }
         active_seq =0;
@@ -1031,7 +1217,7 @@ struct msa* read_clu(struct in_buffer* b , struct msa* msa)
                 //line_len = strnlen(line, BUFFER_LEN);
                 //line_len = nread;
                 //line[line_len-1] = 0;
-                line_len--;     /* last character is newline  */
+                /* line_len--;     /\* last character is newline  *\/ */
                 if(!line_len){
                         active_seq = 0;
                 }else{
@@ -1057,13 +1243,12 @@ struct msa* read_clu(struct in_buffer* b , struct msa* msa)
                                 for(i = j;i < line_len;i++){
                                         msa->letter_freq[(int)p[i]]++;
                                         if(isalpha((int)p[i])){
+                                                seq_ptr->seq[seq_ptr->len] = p[i];
+                                                seq_ptr->len++;
                                                 if(seq_ptr->alloc_len == seq_ptr->len){
                                                         resize_msa_seq(seq_ptr);
                                                 }
-                                                seq_ptr->seq[seq_ptr->len] = p[i];
-                                                seq_ptr->len++;
-                                        }
-                                        if(ispunct((int)p[i])){
+                                        }else if(ispunct((int)p[i])){
                                                 seq_ptr->gaps[seq_ptr->len]++;
                                         }
                                 }
@@ -1077,18 +1262,18 @@ struct msa* read_clu(struct in_buffer* b , struct msa* msa)
         }
         RUN(null_terminate_sequences(msa));
 
-        //fclose(f_ptr);
-        //MFREE(line);
-        return msa;
+        *m = msa;
+        return OK;
 ERROR:
         free_msa(msa);
-        return NULL;
+        return FAIL;
 }
 
 
 
-struct msa* read_msf(struct in_buffer* b,struct msa* msa)
+int read_msf(struct in_buffer* b,struct msa** m)
 {
+        struct msa* msa = NULL;
         struct msa_seq* seq_ptr = NULL;
         char* line = NULL;
         int line_len;
@@ -1103,7 +1288,7 @@ struct msa* read_msf(struct in_buffer* b,struct msa* msa)
                 line = b->l[nl]->line;
                 line_len = b->l[nl]->len;
                 li++;
-                line_len--;     /* last character is newline  */
+                /* line_len--;     /\* last character is newline  *\/ */
                 //fprintf(stdout,"%d \"%s\"\n",line_len,line);
                 if(strstr(line, "//")){
                         //      LOG_MSG("Header done");
@@ -1139,7 +1324,7 @@ struct msa* read_msf(struct in_buffer* b,struct msa* msa)
         for(nl = li; nl < b->n_lines;nl++){
                 line = b->l[nl]->line;
                 line_len = b->l[nl]->len;
-                line_len--;     /* last character is newline  */
+                /* line_len--;     /\* last character is newline  *\/ */
                 if(!line_len){
                         active_seq = 0;
                 }else{
@@ -1154,13 +1339,13 @@ struct msa* read_msf(struct in_buffer* b,struct msa* msa)
                                 for(i = 0;i < line_len-j;i++){
                                         msa->letter_freq[(int)p[i]]++;
                                         if(isalpha((int)p[i])){
+
+                                                seq_ptr->seq[seq_ptr->len] = p[i];
+                                                seq_ptr->len++;
                                                 if(seq_ptr->alloc_len == seq_ptr->len){
                                                         resize_msa_seq(seq_ptr);
                                                 }
-                                                seq_ptr->seq[seq_ptr->len] = p[i];
-                                                seq_ptr->len++;
-                                        }
-                                        if(ispunct((int)p[i])){
+                                        }else if(ispunct((int)p[i])){
                                                 seq_ptr->gaps[seq_ptr->len]++;
                                         }
                                 }
@@ -1174,17 +1359,18 @@ struct msa* read_msf(struct in_buffer* b,struct msa* msa)
         }
         RUN(null_terminate_sequences(msa));
 
+        *m = msa;
         //fclose(f_ptr);
         //MFREE(line);
-        return msa;
+        return OK;
 ERROR:
         free_msa(msa);
-        return NULL;
+        return FAIL;
 }
 
-struct msa* read_fasta( struct in_buffer* b,struct msa* msa)
+int read_fasta( struct in_buffer* b,struct msa** m)
 {
-        //struct msa* msa = NULL;
+        struct msa* msa = NULL;
         struct msa_seq* seq_ptr = NULL;
         //FILE* f_ptr = NULL;
         char* line = NULL;
@@ -1214,14 +1400,14 @@ struct msa* read_fasta( struct in_buffer* b,struct msa* msa)
                 //while(fgets(line, BUFFER_LEN, f_ptr)){
                 //line_len = nread;
 
-                //fprintf(stdout,"%d %d %s",line_len,nread,line);
+                //fprintf(stdout,"%d %s\n",line_len,line);
                 if(line[0] == '>'){
                         /* alloc seq if buffer is full */
                         if(msa->alloc_numseq == msa->numseq){
                                 RUN(resize_msa(msa));
                         }
 
-                        line[line_len-1] = 0;
+                        /* line[line_len-1] = 0; */
                         for(i =0 ; i < line_len;i++){
                                 if(isspace(line[i])){
                                         line[i] = 0;
@@ -1244,17 +1430,19 @@ struct msa* read_fasta( struct in_buffer* b,struct msa* msa)
 
                                         seq_ptr->seq[seq_ptr->len] = line[i];
                                         seq_ptr->len++;
-                                }
-                                if(ispunct((int)line[i])){
+                                }else if(ispunct((int)line[i])){
                                         seq_ptr->gaps[seq_ptr->len]++;
                                 }
                         }
                 }
         }
         RUN(null_terminate_sequences(msa));
+
+
+        *m = msa;
         //fclose(f_ptr);
         //MFREE(line);
-        return msa;
+        return OK;
 ERROR:
         free_msa(msa);
         //if(line){
@@ -1263,7 +1451,7 @@ ERROR:
         //if(f_ptr){
         //fclose(f_ptr);
         //}
-        return NULL;
+        return FAIL;
 }
 
 int merge_msa(struct msa** dest, struct msa* src)
@@ -1274,8 +1462,7 @@ int merge_msa(struct msa** dest, struct msa* src)
         if(d == NULL){
                 d = alloc_msa();
         }
-
-        if(d->L != 0){
+        if(d->L != ALPHA_UNDEFINED){
                 if(d->L != src->L){
                         ERROR_MSG("Input alignments have different alphabets");
                 }
@@ -1361,7 +1548,7 @@ struct msa* alloc_msa(void)
         msa->alloc_numseq = 512;
         msa->numseq = 0;
         msa->num_profiles = 0;
-        msa->L = 0;
+        msa->L = ALPHA_UNDEFINED;
         msa->aligned = 0;
         msa->plen = NULL;
         msa->sip = NULL;
@@ -1469,7 +1656,7 @@ int resize_msa_seq(struct msa_seq* seq)
         MREALLOC(seq->s, sizeof(uint8_t) * seq->alloc_len);
         MREALLOC(seq->gaps, sizeof(int) * (seq->alloc_len+1));
 
-        for(i = old_len;i < seq->alloc_len+1;i++){
+        for(i = old_len+1;i < seq->alloc_len+1;i++){
                 seq->gaps[i] = 0;
         }
 
@@ -1609,7 +1796,7 @@ int read_file_stdin(struct in_buffer** buffer,char* infile)
         char* tmp = NULL;
         size_t b_len = 0;
         ssize_t nread;
-
+        int i;
         //char line[BUFFER_LEN];
         int line_len;
 
@@ -1630,15 +1817,24 @@ int read_file_stdin(struct in_buffer** buffer,char* infile)
         b->n_lines = 0;
 
         while ((nread = getline(&line, &b_len, f_ptr)) != -1){
+                //LOG_MSG("Read %d ", nread);
                 //while(fgets(line, BUFFER_LEN, f_ptr)){
                 line_len = nread;
                 //tmp = b->l[b->n_lines]->line;
                 tmp= NULL;
-                MMALLOC(tmp, sizeof(char) * (line_len));
-                memcpy(tmp, line, line_len);
-                tmp[line_len-1] = 0;
+                MMALLOC(tmp, sizeof(char) * (line_len+1));
+                for(i = 0; i < line_len;i++){
+
+                        if(iscntrl((int) line[i])){
+                                break;
+                        }
+                        tmp[i] = line[i];
+                }
+                tmp[i] = 0;
+                //memcpy(tmp, line, line_len);
+                //tmp[line_len-1] = 0;
                 b->l[b->n_lines]->line = tmp;
-                b->l[b->n_lines]->len = line_len;
+                b->l[b->n_lines]->len = i;
                 b->n_lines++;
                 if(b->n_lines == b->alloc_lines){
                         RUN(resize_in_buffer(b));
@@ -1709,3 +1905,29 @@ void free_in_buffer(struct in_buffer* b)
                 MFREE(b);
         }
 }
+
+
+int sort_by_name(const void *a, const void *b)
+{
+        struct sort_struct_name_chksum* const *one = a;
+        struct sort_struct_name_chksum* const *two = b;
+
+        if(strncmp(*(*one)->name, *(*two)->name,MSA_NAME_LEN) < 0){
+                return -1;
+        }else{
+                return 1;
+        }
+}
+
+
+int sort_by_chksum(const void *a, const void *b)
+{
+        struct sort_struct_name_chksum* const *one = a;
+        struct sort_struct_name_chksum* const *two = b;
+
+        if((*one)->chksum > (*two)->chksum){
+                return -1;
+        }else{
+                return 1;
+        }
+}
