@@ -24,8 +24,13 @@
 #define WEAVE_ALIGNMENT_H
 
 #include "global.h"
+#include "aln_task.h"
 #include "msa.h"
-extern int weave(struct msa* msa, int** map, int* tree);
+
+//extern int weave(struct msa* msa, int** map, int* tree);
+
+extern int weave(struct msa* msa,struct aln_tasks*t);
+extern int make_seq(struct msa* msa,int a,int b,int* path);
 
 extern int clean_aln(struct msa* msa);
 
